/*
** Module   : EnvironmentDaemon.java
** Abstract : Client side daemon to handle Progress 4GL compatible Winodws environment handling
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130604 Created initial version.
** 002 EVL 20140325 Integrating the interface implementation to inspect some system properties on
**                  the client side.
*/

package com.goldencode.p2j.util;

import java.io.*;
import java.util.*;
import com.goldencode.p2j.net.*;

/**
 * A client side class to manipulate with environments. The environment can be the INI file or
 * registry sections containing the keys.
 *
 * @author    EVL
 */
public class EnvironmentDaemon
implements EnvironmentAccessor,
           OperatingSystemInspector
{
   /** Token used to authenticate with the dispatcher when registering APIs. */
   private static Object modToken = null;
   
   /** The default environment to use when load() and use() was not specified. */
   private EnvironmentReader defaultEnv = null;
   
   /** The currently loaded environment to use instead of default. */
   private EnvironmentReader currentEnv = null;
   
   /** Map to store environments fo possible reuse. */
   private Map<String, EnvironmentReader> envMap = new HashMap<String, EnvironmentReader>();
   
   /**
    * Create an instance and export its API to the network.
    *
    * @param    single
    *           <code>true</code> to startup within the server process which must bypass the
    *           shared infrastructure initialization. Use <code>false</code> for the normal
    *           client JVM startup.
    */
   public EnvironmentDaemon(boolean single)
   {
      defaultEnv = new Registry();
      
      synchronized (EnvironmentDaemon.class)
      {
         modToken = RemoteObject.registerServer(EnvironmentAccessor.class,
                                                this,
                                                modToken,
                                                single);
         // Register tool to check the client OS specifics
         RemoteObject.registerServer(OperatingSystemInspector.class, this, modToken, single);
      }
   }
      
   /**
    * Creates application defaults implementing Progress LOAD statement.
    * 
    * @param env 
    *        The environment to create or load. Can be the registry key, initialization file.
    * @param directory 
    *        The directory path for existing initialization file or place to create
    *        initialization file.
    * @param flagNew
    *        If <code>true</code> - the new file or registry key will be created overwriting the
    *        possibly existed one.
    * @param baseKey
    *        When the parameter is specified the serching is performed under the particular key
    *        value. It is possible to specify "INI" as parameter. In this case search or creation
    *        will be done inside initialization file specified in env parameter.
    */
   public void load(String env, String directory, boolean flagNew, String baseKey)
   {
      EnvironmentReader envAcc = null;
      if (baseKey != null && baseKey.equals("INI"))
      {
         envAcc = new StanzaIni(env, directory, flagNew);
      }
      else
      {
         envAcc = new Registry(env, flagNew, baseKey);
      }
      envAcc.load();
      envMap.put(env.toLowerCase(), envAcc);
   }

   /**
    * Specifies environment defaults to use in subsequent windows.
    * 
    * @param env 
    *        The environment to load. If the parameter is empty string ("") the default
    *        environment becomes the current one.
    */
   public void use(String env)
   {
      // Empty string as environment makes the default environment the current one
      if (env == null || env.isEmpty())
      {
         currentEnv = null;
      }
      else
      {
         currentEnv = envMap.get(env.toLowerCase());
      }
   }

   /**
    * Adds, modifies and deletes keys in current environment.
    * 
    * @param section 
    *        The name of the section containing the key to modify.
    * @param key 
    *        The name of the key key to modify or default key if not specified.
    * @param value 
    *        The new value of the key under modification.
    */
   public void setKeyValue(String section, String key, String value)
   {
      if (section == null || value == null)
      {
         return;
      }
      if (currentEnv != null)
      {
         currentEnv.setKeyValue(section, key, value);
      }
      else
      {
         defaultEnv.setKeyValue(section, key, value);
      }
   }

   /**
    * Getting the key value from the current environment.
    * 
    * @param section 
    *        The name of the section containing the key to get.
    * @param key 
    *        The name of the key key to get or default key if not specified.
    * 
    * @return The value of the key.
    */
   public String getKeyValue(String section, String key)
   {
      if (section == null)
      {
         return new String("");
      }
      if (currentEnv != null)
      {
         return currentEnv.getKeyValue(section, key);
      }
      else
      {
         return defaultEnv.getKeyValue(section, key);
      }
   }

   /**
    * Unloads specifies environment from the current one.
    * 
    * @param env 
    *        The environment to unload. Can be the registry or initialization file. The
    *        environment to unload should be previously loaded by load() method.
    */
   public void unload(String env)
   {
      if (currentEnv != null)
      {
         currentEnv.unload();
         currentEnv = null;
      }
      return;
   }

   /**
    * If there is still no match, then any <code>os.name</code> with the
    * lowercase text "unix" will return "UNIX" and any <code>os.name</code>
    * with the lowercase text "windows" will return "WIN32". 
    * <p>
    * If there is still no match, the <code>os.name</code> returned by the
    * Java will be returned. 
    *
    * @return   The name of the current operating system.
    */
   public String getOperatingSystem()
   {
      return System.getProperty("os.name").toLowerCase();
   }

   /**
    * Gets the path separator character used to parse the Progress search
    * path.
    * <p>
    * The value returned may have been found via a search algorithm that
    * is account (user or process) specific or group specific
    * within the current server or a global default for all servers. 
    * <p>
    * If no value is found via this lookup, then the default value of
    * ':' will be returned.
    *
    * @return   The path to Progress source files.
    */
   public String getPathSeparator()
   {
      return File.pathSeparator;
   }

   /**
    * Checks is the current OS the P2J is running on belongs to the Windows
    * family.
    *
    * @return   <code>true</code> if the current OS is from Windows family or 
    *           <code>false</code> otherwise.
    */
   public logical isUnderWindowsFamily()
   {
      return new logical(System.getProperty("os.name").toLowerCase().indexOf("win") >= 0);
   }
}
